/*-
 * Copyright 2006 John-Mark Gurney.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: //depot/skiplist/main/main.c#3 $
 *
 */

#include <skiplist.h>

#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void printtimevaldiff(struct timeval a, struct timeval b);
int intcmp(void *a, void *b);

void
printtimevaldiff(struct timeval a, struct timeval b)
{
	int dsec;
	int dusec;

	dsec = a.tv_sec - b.tv_sec;
	dusec = a.tv_usec - b.tv_usec;

	if (dusec < 0) {
		dsec--;
		dusec += 1000000;
	}
	printf("%d.%0.6d", dsec, dusec);
}

int
intcmp(void *a, void *b)
{
	unsigned int x = (unsigned int) a;
	unsigned int y = (unsigned int) b;
	if (x < y)
		return -1;
	if (x == y)
		return 0;
	if (x > y)
		return 1;

	abort();
	/* NOTREACHED */
	return 0;
}

int
main(int argc, char **argv)
{
	int i;
	struct skiplist *bt;

#if 0
	if (argc != 2) {
		printf("usage: test <nodesize>\n");
		exit(1);
	}

	bt = createskiplist(intcmp, atoi(argv[1]), 1, 0, UINT_MAX);
	printf("original tree:\n");
	printskiplist(bt);

	for (i = 0; i < 1000000; i++)
		insertkey(bt, random());
	/*dumptreestats(bt);*/
#elif 1
	int *nums;
	int t;
	struct rusage start, end;

	if (argc != 3) {
		printf("usage: test <numnodes> <nodesize>\n");
		exit(1);
	}

	t = atoi(argv[1]);
	nums = malloc(sizeof *nums * t);
	bt = createskiplist(intcmp, atoi(argv[2]), 1, (void *)0,
	    (void *)UINT_MAX);

	getrusage(RUSAGE_SELF, &start);
	for (i = 0; i < t; i++)
		insertkey(bt, (void *)(nums[i] = random()));
	getrusage(RUSAGE_SELF, &end);

	printf("insert time: ");
	printtimevaldiff(end.ru_utime, start.ru_utime);
	puts("");

	getrusage(RUSAGE_SELF, &start);
	for (i = 0; i < t; i++)
		searchkey(bt, (void *)nums[random() % t]);
	getrusage(RUSAGE_SELF, &end);

	printf("search time: ");
	printtimevaldiff(end.ru_utime, start.ru_utime);
	puts("");

	getrusage(RUSAGE_SELF, &start);
	for (i = 0; i < t; i++)
		if (!deletekey(bt, (void *)nums[i]))
			printf("failed to delete %p.\n", (void *)nums[i]);
	getrusage(RUSAGE_SELF, &end);

	printf("delete time: ");
	printtimevaldiff(end.ru_utime, start.ru_utime);
	puts("");
#else
	int *nums;
	int t;

	if (argc != 2) {
		printf("usage: test <numnodes>\n");
		exit(1);
	}

	t = atoi(argv[1]);
	nums = malloc(sizeof *nums * t);
	bt = createskiplist(intcmp, 16, 1, 0, UINT_MAX);
	printskiplist(bt);

	for (i = 0; i < t; i++) {
		insertkey(bt, (nums[i] = random()));
		/*if (!checkbtree(bt, 0x0, 0xffffffff)) {
			printf("tree isn't in order after insert!!, at %d:%p\n",
			    i, nums[i]);
			dumptree(bt);
			exit(1);
		}*/
	}
	printskiplist(bt);
	for (i = 0; i < t; i++) {
		printf("deleting %p\n", nums[i]);
		if (!deletekey(bt, nums[i]))
			printf("failed to delete %p.\n", nums[i]);
		/*if (!checkbtree(bt, 0x0, 0xffffffff)) {
			printf("tree isn't in order after del!!, at %d:%p\n", i,
			    nums[i]);
			dumptree(bt);
			exit(1);
		}
		dumptree(bt);*/
		printskiplist(bt);
	}
#endif
	exit(0);
	/* NOTREACHED */
	return 0;
}
